@@ -1,6 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2001, 2002, 2003, 2004, 
-   2007 Free Software Foundation, Inc.
+   Copyright (C) 1999-2023 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -13,9 +12,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with GNU Mailutils; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
-   MA 02110-1301 USA */
+   along with GNU Mailutils.  If not, see <http://www.gnu.org/licenses/>. */
 
 /*
   GSSAPI authentication for imap (rfc 1731). 
@@ -64,11 +61,11 @@ display_status_1 (char *m, OM_uint32 code, int type)
 				     type, GSS_C_NO_OID, &msg_ctx, &msg);
       if (GSS_ERROR (maj_stat))
 	{
-	  asprintf ((char**)&msg.value, "code %d", code);
+	  mu_asprintf ((char**)&msg.value, "code %d", code);
 	  msg.length = strlen (msg.value);
 	}
 
-      syslog (LOG_ERR, _("GSS-API error %s (%s): %.*s"),
+      mu_diag_output (MU_DIAG_ERROR, _("GSS-API error %s (%s): %.*s"),
 	      m, type == GSS_C_GSS_CODE ? _("major") : _("minor"),
 	      (int) msg.length, (char *) msg.value);
 
@@ -108,18 +105,19 @@ imap4d_gss_userok (gss_buffer_t client_name, char *name)
 }
 #endif
 
-static int
-auth_gssapi (struct imap4d_command *command,
-	     char *auth_type_unused, char *arg_unused, char **username)
+static enum imap4d_auth_result
+auth_gssapi (struct imap4d_auth *ap)
 {
   gss_buffer_desc tokbuf, outbuf;
   OM_uint32 maj_stat, min_stat, min_stat2;
-  OM_uint32 cflags;
+  int cflags;
   OM_uint32 sec_level, mech;
   gss_ctx_id_t context;
   gss_cred_id_t cred_handle, server_creds;
   gss_OID mech_type;
-  char *token_str;
+  char *token_str = NULL;
+  size_t token_size = 0;
+  size_t token_len;
   unsigned char *tmp = NULL;
   size_t size;
   gss_name_t server_name;
@@ -137,7 +135,7 @@ auth_gssapi (struct imap4d_command *command,
      work (possibly due to a bug in krb5_gss_accept_sec_context()), so
      we acquire server credentials explicitly. */
 
-  asprintf ((char **) &tmp, "imap@%s", util_localname ());
+  mu_asprintf ((char **) &tmp, "imap@%s", util_localname ());
   tokbuf.value = tmp;
   tokbuf.length = strlen (tokbuf.value) + 1;
   maj_stat = gss_import_name (&min_stat, &tokbuf,
@@ -145,7 +143,8 @@ auth_gssapi (struct imap4d_command *command,
   if (maj_stat != GSS_S_COMPLETE)
     {
       display_status ("import name", maj_stat, min_stat);
-      return RESP_NO;
+      ap->response = RESP_NO;
+      return imap4d_auth_resp;
     }
 
   maj_stat = gss_acquire_cred (&min_stat, server_name, 0,
@@ -156,23 +155,25 @@ auth_gssapi (struct imap4d_command *command,
   if (maj_stat != GSS_S_COMPLETE)
     {
       display_status ("acquire credentials", maj_stat, min_stat);
-      return RESP_NO;
+      ap->response = RESP_NO;
+      return imap4d_auth_resp;
     }
 
   /* Start the dialogue */
 
-  util_send ("+ \r\n");
-  util_flush_output ();
+  io_sendf ("+ \n");
+  io_flush ();
   
   context = GSS_C_NO_CONTEXT;
 
   for (;;)
     {
-      token_str = imap4d_readline_ex ();
-      util_base64_decode (token_str, strlen (token_str), &tmp, &size);
+      OM_uint32 ret_flags;
+      
+      io_getline (&token_str, &token_size, &token_len);
+      mu_base64_decode ((unsigned char*) token_str, token_len, &tmp, &size);
       tokbuf.value = tmp;
       tokbuf.length = size;
-      free (token_str);
 
       maj_stat = gss_accept_sec_context (&min_stat,
 					 &context,
@@ -182,14 +183,14 @@ auth_gssapi (struct imap4d_command *command,
 					 &client,
 					 &mech_type,
 					 &outbuf,
-					 &cflags, NULL, &cred_handle);
+					 &ret_flags, NULL, &cred_handle);
       free (tmp);
       if (maj_stat == GSS_S_CONTINUE_NEEDED)
 	{
 	  if (outbuf.length)
 	    {
-	      util_base64_encode (outbuf.value, outbuf.length, &tmp, &size);
-	      util_send ("+ %*.*s\r\n", size, size, tmp);
+	      mu_base64_encode (outbuf.value, outbuf.length, &tmp, &size);
+	      io_sendf ("+ %s\n", tmp);
 	      free (tmp);
 	      gss_release_buffer (&min_stat, &outbuf);
 	    }
@@ -202,17 +203,18 @@ auth_gssapi (struct imap4d_command *command,
       display_status ("accept context", maj_stat, min_stat);
       maj_stat = gss_delete_sec_context (&min_stat, &context, &outbuf);
       gss_release_buffer (&min_stat, &outbuf);
-      return RESP_NO;
+      free (token_str);
+      ap->response = RESP_NO;
+      return imap4d_auth_resp;
     }
 
   if (outbuf.length)
     {
-      util_base64_encode (outbuf.value, outbuf.length, &tmp, &size);
-      util_send ("+ %*.*s\r\n", size, size, tmp);
+      mu_base64_encode (outbuf.value, outbuf.length, &tmp, &size);
+      io_sendf ("+ %s\n", tmp);
       free (tmp);
       gss_release_buffer (&min_stat, &outbuf);
-      token_str = imap4d_readline_ex ();
-      free (token_str);
+      io_getline (&token_str, &token_size, &token_len);
     }
 
   /* Construct security-level data */
@@ -224,16 +226,18 @@ auth_gssapi (struct imap4d_command *command,
   if (maj_stat != GSS_S_COMPLETE)
     {
       display_status ("wrap", maj_stat, min_stat);
-      return RESP_NO;
+      free (token_str);
+      ap->response = RESP_NO;
+      return imap4d_auth_resp;
     }
   
-  util_base64_encode (outbuf.value, outbuf.length, &tmp, &size);
-  util_send ("+ %*.*s\r\n", size, size, tmp);
+  mu_base64_encode (outbuf.value, outbuf.length, &tmp, &size);
+  io_sendf ("+ %s\n", tmp);
   free (tmp);
 
-  token_str = imap4d_readline_ex ();
-  util_base64_decode (token_str, strlen (token_str),
-		      (unsigned char **) &tokbuf.value, &tokbuf.length);
+  io_getline (&token_str, &token_size, &token_len);
+  mu_base64_decode ((unsigned char *) token_str, token_len,
+		    (unsigned char **) &tokbuf.value, &tokbuf.length);
   free (token_str);
 
   maj_stat = gss_unwrap (&min_stat, context, &tokbuf, &outbuf, &cflags,
@@ -242,7 +246,8 @@ auth_gssapi (struct imap4d_command *command,
   if (maj_stat != GSS_S_COMPLETE)
     {
       display_status ("unwrap", maj_stat, min_stat);
-      return RESP_NO;
+      ap->response = RESP_NO;
+      return imap4d_auth_resp;
     }
   
   sec_level = ntohl (*(OM_uint32 *) outbuf.value);
@@ -251,29 +256,31 @@ auth_gssapi (struct imap4d_command *command,
   mech = sec_level >> 24;
   if ((mech & SUPPORTED_P_MECH) == 0)
     {
-      syslog (LOG_NOTICE,
-	      _("Client requested unsupported protection mechanism (%d)"),
+      mu_diag_output (MU_DIAG_NOTICE,
+	      _("client requested unsupported protection mechanism (%d)"),
 	      mech);
       gss_release_buffer (&min_stat, &outbuf);
       maj_stat = gss_delete_sec_context (&min_stat, &context, &outbuf);
       gss_release_buffer (&min_stat, &outbuf);
-      return RESP_NO;
+      ap->response = RESP_NO;
+      return imap4d_auth_resp;
     }
   protection_mech = mech;
   client_buffer_size = sec_level & 0x00ffffffff;
 
-  *username = malloc (outbuf.length - 4 + 1);
-  if (!*username)
+  ap->username = malloc (outbuf.length - 4 + 1);
+  if (!ap->username)
     {
-      syslog (LOG_NOTICE, _("Not enough memory"));
+      mu_diag_output (MU_DIAG_NOTICE, _("not enough memory"));
       gss_release_buffer (&min_stat, &outbuf);
       maj_stat = gss_delete_sec_context (&min_stat, &context, &outbuf);
       gss_release_buffer (&min_stat, &outbuf);
-      return RESP_NO;
+      ap->response = RESP_NO;
+      return imap4d_auth_resp;
     }
        
-  memcpy (*username, (char *) outbuf.value + 4, outbuf.length - 4);
-  (*username)[outbuf.length - 4] = '\0';
+  memcpy (ap->username, (char *) outbuf.value + 4, outbuf.length - 4);
+  ap->username[outbuf.length - 4] = '\0';
   gss_release_buffer (&min_stat, &outbuf);
 
   maj_stat = gss_display_name (&min_stat, client, &client_name, &mech_type);
@@ -282,36 +289,40 @@ auth_gssapi (struct imap4d_command *command,
       display_status ("get client name", maj_stat, min_stat);
       maj_stat = gss_delete_sec_context (&min_stat, &context, &outbuf);
       gss_release_buffer (&min_stat, &outbuf);
-      free (*username);
-      return RESP_NO;
+      free (ap->username);
+      ap->response = RESP_NO;
+      return imap4d_auth_resp;
     }
 
 #ifdef WITH_GSS
-  baduser = !gss_userok (client, *username);
+  baduser = !gss_userok (client, ap->username);
 #else
-  baduser = imap4d_gss_userok (&client_name, *username);
+  baduser = imap4d_gss_userok (&client_name, ap->username);
 #endif
 
   if (baduser)
     {
-      syslog (LOG_NOTICE, _("GSSAPI user %s is NOT authorized as %s"),
-	      (char *) client_name.value, *username);
+      mu_diag_output (MU_DIAG_NOTICE,
+		      _("GSSAPI user %s is NOT authorized as %s"),
+		      (char *) client_name.value, ap->username);
       maj_stat = gss_delete_sec_context (&min_stat, &context, &outbuf);
       gss_release_buffer (&min_stat, &outbuf);
       gss_release_buffer (&min_stat, &client_name);
-      free (*username);
-      return RESP_NO;
+      free (ap->username);
+      ap->response = RESP_NO;
+      return imap4d_auth_resp;
     }
   else
     {
-      syslog (LOG_NOTICE, _("GSSAPI user %s is authorized as %s"),
-	      (char *) client_name.value, *username);
+      mu_diag_output (MU_DIAG_NOTICE, _("GSSAPI user %s is authorized as %s"),
+	      (char *) client_name.value, ap->username);
     }
 
   gss_release_buffer (&min_stat, &client_name);
   maj_stat = gss_delete_sec_context (&min_stat, &context, &outbuf);
   gss_release_buffer (&min_stat, &outbuf);
-  return RESP_OK;
+  ap->response = RESP_OK;
+  return imap4d_auth_resp;
 }
 
 void
