@@ -1,5 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2008-2021 Free Software Foundation, Inc.
+   Copyright (C) 2008-2023 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -15,7 +15,7 @@
    along with GNU Mailutils.  If not, see <http://www.gnu.org/licenses/>. */
 
 int mu_switch_to_privs (uid_t uid, gid_t gid, mu_list_t retain_groups);
-void mu_set_signals (RETSIGTYPE (*handler) (int signo), int *sigv, int sigc);
+void mu_set_signals (void (*handler) (int signo), int *sigv, int sigc);
 const char *mu_strexit (int code);
 
 extern int manlock_mandatory_locking;
