@@ -1,5 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2008-2019 Free Software Foundation, Inc.
+   Copyright (C) 2008-2023 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -21,7 +21,7 @@
 #include <signal.h>
 
 void
-mu_set_signals (RETSIGTYPE (*handler) (int signo), int *sigv, int sigc)
+mu_set_signals (void (*handler) (int signo), int *sigv, int sigc)
 {
   int i;
   struct sigaction act;
