@@ -1,6 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999-2001, 2005, 2007, 2009-2012, 2014-2017 Free
-   Software Foundation, Inc.
+   Copyright (C) 1999-2023 Free Software Foundation, Inc.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -120,7 +119,7 @@ void
 mu_diag_vprintf (int level, const char *fmt, va_list ap)
 {
   mu_diag_init ();
-  mu_stream_printf (mu_strerr, "\033<%d>", level);
+  mu_stream_printf (mu_strerr, "\033s<%d>", level);
   mu_stream_vprintf (mu_strerr, fmt, ap);
 }
 
