@@ -1,5 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2010-2012, 2014-2018 Free Software Foundation, Inc.
+   Copyright (C) 2010-2023 Free Software Foundation, Inc.
 
    This library is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
@@ -127,7 +127,7 @@ lr_set_file (struct mu_locus_range *loc, char const *fname, unsigned len,
      FfLlCc
 
    The bits f, l, and c (file, line, and column) are toggled cyclically.
-   The value 0 means locus beg, 1 meand locus end.
+   The value 0 means locus beg, 1 means locus end.
    The bits F, L, and C are set once and indicate that the corresponding
    bit was toggled at least once.
  */
@@ -272,6 +272,13 @@ _log_write (struct _mu_stream *str, const char *buf, size_t size,
 	  mu_stream_print_locus_range (sp->transport, &loc);
   	  mu_stream_write (sp->transport, ": ", 2, NULL);
 	}
+
+      if ((logmode & MU_LOGMODE_PREFIX) && sp->prefix)
+	{
+	  mu_stream_write (sp->transport, sp->prefix, strlen (sp->prefix),
+			   NULL);
+  	  mu_stream_write (sp->transport, ": ", 2, NULL);
+	}
       
       if ((logmode & MU_LOGMODE_SEVERITY) &&
 	  !(sp->sevmask & MU_DEBUG_LEVEL_MASK (severity)))
@@ -302,6 +309,7 @@ _log_done (struct _mu_stream *str)
 {
   struct _mu_log_stream *sp = (struct _mu_log_stream *)str;
   mu_locus_range_deinit (&sp->locrange);
+  mu_ident_deref (sp->prefix);
   mu_stream_destroy (&sp->transport);
 }
 
@@ -475,48 +483,6 @@ _log_ctl (struct _mu_stream *str, int code, int opcode, void *arg)
 	      lr->end.mu_col = sp->locrange.end.mu_col;
 	    }
 	  break;
-	  
-	case MU_IOCTL_LOGSTREAM_GET_LOCUS_DEPRECATED:
-	  if (!arg)
-	    return EINVAL;
-	  else
-	    {
-	      struct mu_locus_DEPRECATED *ploc = arg;
-	      if (sp->locrange.beg.mu_file)
-		{
-		  ploc->mu_file = strdup (sp->locrange.beg.mu_file);
-		  if (!ploc->mu_file)
-		    return ENOMEM;
-		}
-	      else
-		ploc->mu_file = NULL;
-	      ploc->mu_line = sp->locrange.beg.mu_line;
-	      ploc->mu_col = sp->locrange.beg.mu_col;
-	    }
-	  break;
-	
-	case MU_IOCTL_LOGSTREAM_SET_LOCUS_DEPRECATED:
-	  {
-	    struct mu_locus_DEPRECATED *ploc = arg;
-
-	    mu_ident_deref (sp->locrange.end.mu_file);
-	    sp->locrange.end.mu_file = NULL;
-	    if (arg)
-	      {
-		status = lr_set_file (&sp->locrange, ploc->mu_file, 0, 0);
-		if (status)
-		  return status;
-		lr_set_line (&sp->locrange, ploc->mu_line, 0);
-		lr_set_col (&sp->locrange, ploc->mu_col, 0);
-	      }
-	    else
-	      {
-		mu_ident_deref (sp->locrange.beg.mu_file);
-		sp->locrange.beg.mu_file = NULL;
-	      }
-	    
-	    break;
-	  }
 
 	case MU_IOCTL_LOGSTREAM_SET_LOCUS_LINE:
 	  if (!arg)
@@ -590,10 +556,39 @@ _log_ctl (struct _mu_stream *str, int code, int opcode, void *arg)
 			    &sp->locrange.beg.mu_file);
 	      mu_ident_ref (sp->locrange.end.mu_file,
 			    &sp->locrange.end.mu_file);
-	      
+	      mu_ident_ref (sp->prefix, &newp->prefix);
 	      *(mu_stream_t*) arg = str;
 	    }
 	  break;
+
+	case MU_IOCTL_LOGSTREAM_GET_PREFIX:
+	  if (!arg)
+	    return EINVAL;
+	  else
+	    {
+	      char **ret_ptr = arg;
+
+	      if (!sp->prefix)
+		sp->prefix = NULL;
+	      else
+		{
+		  char *copy = strdup (sp->prefix);
+		  if (!copy)
+		    return errno;
+		  *ret_ptr = copy;
+		}
+	    }
+	  break;
+	      
+	case MU_IOCTL_LOGSTREAM_SET_PREFIX:
+	  {
+	    mu_ident_deref (sp->prefix);
+	    if (arg)
+	      mu_ident_ref (arg, &sp->prefix);
+	    else
+	      sp->prefix = NULL;
+	  }
+	  break;
 	  
 	default:
 	  return EINVAL;
@@ -640,33 +635,5 @@ mu_log_stream_create (mu_stream_t *pstr, mu_stream_t transport)
   
   return 0;
 }
-
-static char wiki_url[] = "http://mailutils.org/wiki/Source_location_API#Deprecated_interface";
-
-int
-mu_ioctl_logstream_get_locus_deprecated (void)
-{
-  static int warned;
-  if (!warned)
-    {
-      mu_error (_("the program uses MU_IOCTL_LOGSTREAM_GET_LOCUS, which is deprecated"));
-      mu_error (_("please see %s, for detailed guidelines"), wiki_url);
-      warned = 1;
-    }
-  return MU_IOCTL_LOGSTREAM_GET_LOCUS_DEPRECATED;
-}
-
-int
-mu_ioctl_logstream_set_locus_deprecated (void)
-{
-  static int warned;
-  if (!warned)
-    {
-      mu_error (_("program uses MU_IOCTL_LOGSTREAM_SET_LOCUS, which is deprecated"));
-      mu_error (_("please see %s, for detailed guidelines"), wiki_url);
-      warned = 1;
-    }
-  return MU_IOCTL_LOGSTREAM_SET_LOCUS_DEPRECATED;
-}
 
 
