@@ -1,5 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999-2021 Free Software Foundation, Inc.
+   Copyright (C) 1999-2023 Free Software Foundation, Inc.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -256,7 +256,7 @@ struct attrib
 
 /*
  * Both name and value point to the memory allocated in one block with
- * the attrib structure.  Attributes form a singly-linled list with the
+ * the attrib structure.  Attributes form a singly-linked list with the
  * list head pointing to the first attribute defined in the file name.
  *
  * Functions for accessing attribute lists:
@@ -271,7 +271,7 @@ struct attrib
    vlen - Length of the value.
 
    On success, updates head and returns the new head attribute.
-   On memory allocation error, returne NULL and does not modify head.
+   On memory allocation error, return NULL and does not modify head.
 */
 static inline struct attrib *
 attrib_push (struct attrib **head, char const *name, size_t nlen,
@@ -424,7 +424,7 @@ maildir_message_name_parse (char const *name, char **attrnames,
   int state = 0, oldstate;
   char const *endp = p;
   char const *startval;
-  char const *endval;
+  char const *endval = NULL;
   struct attrib *athead = NULL;
   size_t len;
   int f = 0;
@@ -513,7 +513,7 @@ maildir_open (struct _maildir_data *md)
 static void
 maildir_close (struct _maildir_data *md)
 {
-  if (md->folder_fd == -1)
+  if (md->folder_fd != -1)
     {
       close (md->folder_fd);
       md->folder_fd = -1;
@@ -647,7 +647,7 @@ maildir_message_free (struct _amd_message *amsg)
 }
 
 /*
- * Maldir scanning
+ * Maildir scanning
  */
 int
 maildir_tmp_flush (struct _maildir_data *md)
@@ -749,7 +749,7 @@ maildir_subdir_scan (struct _maildir_data *md, int subdir)
  * Maildir attribute fixup
  * =======================
  *
- * Messag info flags used by Mailutils versions prior to 3.10.90 differ
+ * Message info flags used by Mailutils versions prior to 3.10.90 differ
  * from those described in the reference maildir implementation
  * (http://cr.yp.to/proto/maildir.html)
  * This was reported in http://savannah.gnu.org/bugs/?56428.
@@ -762,10 +762,10 @@ maildir_subdir_scan (struct _maildir_data *md, int subdir)
  *  MU_ATTRIBUTE_ANSWERED      R         a
  *  MU_ATTRIBUTE_SEEN          -         S
  *
- * The "Attribute" column lists the maiutils attribute flag, the
+ * The "Attribute" column lists the mailutils attribute flag, the
  * "Reference" column contains the letter that corresponds to that flag
  * in the reference implementation and the "MU-3.10" column shows the
- * letter in mailutils 3.10 and earloer.
+ * letter in mailutils 3.10 and earlier.
  *
  * Starting from version 3.10.90 this discrepancy is fixed.  To make sure
  * both old and new message attributes are correctly recognized, we try 
@@ -1388,20 +1388,20 @@ maildir_create (struct _amd_data *amd, int flags)
 
 static int
 maildir_msg_finish_delivery (struct _amd_data *amd, struct _amd_message *amm,
-			     const mu_message_t orig_msg)
+			     const mu_message_t orig_msg,
+			     mu_attribute_t atr)
 {
   struct _maildir_data *md = (struct _maildir_data *)amd;
   struct _maildir_message *msg = (struct _maildir_message *) amm;
-  mu_attribute_t attr;
   int flags;
   int rc;
   int src_fd = -1, dst_fd = -1;
   struct string_buffer sb = STRING_BUFFER_INITIALIZER;
   char const *newname;
   
-  if (mu_message_get_attribute (orig_msg, &attr) == 0
-      && mu_attribute_is_read (attr)
-      && mu_attribute_get_flags (attr, &flags) == 0)
+  if ((atr || mu_message_get_attribute (orig_msg, &atr) == 0)
+      && mu_attribute_get_flags (atr, &flags) == 0
+      && flags)
     {
       msg->subdir = SUB_CUR;
       rc = string_buffer_format_message_name (&sb, msg, flags);
@@ -1449,6 +1449,19 @@ maildir_msg_finish_delivery (struct _amd_data *amd, struct _amd_message *amm,
 		     amd->name, subdir_name[SUB_TMP], msg->file_name,
 		     mu_strerror (errno)));
 	}
+      
+      if (strcmp (msg->file_name, newname))
+	{
+	  char *p = strdup (newname);
+	  if (!p)
+	    rc = errno;
+	  else
+	    {
+	      free (msg->file_name);
+	      msg->file_name = p;
+	      /* uniq_len remains the same */
+	    }
+	}
     }
   else
     {
