@@ -1,6 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2001, 2005, 2007, 2010-2012, 2014-2017 Free
-   Software Foundation, Inc.
+   Copyright (C) 1999-2023 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -63,7 +62,7 @@ static const struct mail_command_entry mail_command_table[] = {
     mail_file, file_compl },
   { "folders",	"folders",	"folders",			0,
     mail_folders, no_compl },
-  { "fo",	"followup",	"fo[llowup] [msglist]",		EF_SEND,
+  { "fo",	"followup",	"fo[llowup] [message]",		EF_SEND,
     mail_followup, msglist_compl },
   { "f",	"from",		"f[rom] [msglist]",		0,
     mail_from, msglist_compl },
@@ -85,6 +84,8 @@ static const struct mail_command_entry mail_command_table[] = {
     mail_list, no_compl },
   { "m",	"mail",		"m[ail] [address...]",		EF_SEND,
     mail_send, alias_compl },
+  { "M",        "Mail",         "M[ail] [address...]",		EF_SEND,
+    mail_send, alias_compl },
   { "mb",	"mbox",		"mb[ox] [msglist]",		0,
     mail_mbox, msglist_compl },
   { "n",	"next",		"n[ext] [message]",		0,
@@ -96,7 +97,7 @@ static const struct mail_command_entry mail_command_table[] = {
   { "P",	"Print",	"P[rint] [msglist]",		0,
     mail_print, msglist_compl },
   { "pi",	"pipe",		"pi[pe] [[msglist] command]",	0,
-    mail_pipe, no_compl }, /* FIXME: exec_compl */
+    mail_pipe, exec_compl }, 
   { "pre",	"preserve",	"pre[serve] [msglist]",		0,
     mail_hold, msglist_compl },
   { "prev",	"previous",	"prev[ious] [message]",		0,
@@ -109,9 +110,9 @@ static const struct mail_command_entry mail_command_table[] = {
     mail_reply, msglist_compl },
   { "R",	"Respond",	"R[espond] [msglist]",		EF_SEND,
     mail_reply, msglist_compl },
-  { "r",	"reply",	"r[eply] [msglist]",		EF_SEND,
+  { "r",	"reply",	"r[eply] [message]",		EF_SEND,
     mail_reply, msglist_compl },
-  { "r",	"respond",	"r[espond] [msglist]",		EF_SEND,
+  { "r",	"respond",	"r[espond] [message]",		EF_SEND,
     mail_reply, msglist_compl },
   { "ret",	"retain",	"ret[ain] [header-field]",	0,
     mail_retain, no_compl },
@@ -130,7 +131,7 @@ static const struct mail_command_entry mail_command_table[] = {
   { "sete",     "setenv",       "sete[nv] [name[=value]]",      0,
     mail_setenv, no_compl },
   { "sh",	"shell",	"sh[ell] [command]",		0,
-    mail_shell, no_compl }, /* FIXME: exec_compl */
+    mail_shell, shell_compl },
   { "si",	"size",		"si[ze] [msglist]",		0,
     mail_size, msglist_compl },
   { "so",	"source",	"so[urce] file",		0,
@@ -151,6 +152,8 @@ static const struct mail_command_entry mail_command_table[] = {
     mail_print, msglist_compl },
   { "una",	"unalias",	"una[lias] [alias]...",		0,
     mail_unalias, NULL },
+  { "U",        "unread",       "unread [msglist]",             0,
+    mail_unread, msglist_compl },
   { "u",	"undelete",	"u[ndelete] [msglist]",		0,
     mail_undelete, msglist_compl },
   { "unf",      "unfold",       "unf[old] [header-field]",      0,
@@ -178,7 +181,7 @@ static const struct mail_command_entry mail_command_table[] = {
   { "?",	"?",		"? [command...]",		0,
     mail_help, command_compl },
   { "!",	"",		"![command]",			0,
-    mail_shell, exec_compl },
+    mail_shell, shell_compl },
   { "=",	"=",		"=",				0,
     mail_eq, no_compl },
   { "#",	"#",		"# comment",			0,
@@ -188,7 +191,7 @@ static const struct mail_command_entry mail_command_table[] = {
   { "+",	"+",		"+ [message]",			0,
     mail_next, msglist_compl },
   { "|",	"|",		"| [[msglist] command]",	0,
-    mail_pipe, msglist_compl }, /* FIXME: msglist_exec_compl */
+    mail_pipe, exec_compl },
   { "-",	"-",		"- [message]",			0,
     mail_previous, msglist_compl },
 };
@@ -228,6 +231,7 @@ static const struct mail_escape_entry mail_escape_table[] = {
   {"-",	"-",	"-[mail-command]",  escape_command },
   {"+",	"+",	"+name [content-type [encoding]]", escape_attach },
   {"^",	"^",	"^N",               escape_remove_attachment },
+  {"/", "/",    "/",                escape_toggle_multipart_type },
   {"?",	"?",	"?",		    escape_help },
   {"A",	"A",	"A",		    escape_sign },
   {"a",	"a",	"a",		    escape_sign },
