@@ -1,6 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2003, 2005-2012, 2014-2017 Free Software Foundation,
-   Inc.
+   Copyright (C) 2003-2023 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -1048,9 +1047,15 @@ handle_message (mu_message_t msg, msg_part_t part, msg_handler_t fun, void *data
   if (ismime)
     {
       size_t i, nparts;
-
-      mu_message_get_num_parts (msg, &nparts);
-
+      int rc;
+      
+      rc = mu_message_get_num_parts (msg, &nparts);
+      if (rc)
+	{
+	  mu_diag_funcall (MU_DIAG_ERR, "mu_message_get_num_parts", NULL, rc);
+	  return rc;
+	}
+	
       msg_part_incr (part);
       for (i = 1; i <= nparts; i++)
 	{
@@ -1172,7 +1177,7 @@ list_handler (mu_message_t msg, msg_part_t part, char *type, char *encoding,
       get_extbody_params (msg, &content_type, &content_descr);
 	
       printf ("          ");
-      printf ("%-25s", content_type ? content_type : "");
+      printf ("%-25s", mu_prstr (content_type));
       if (content_descr)
 	printf ("       %s", content_descr);
       printf ("\n");
@@ -1322,7 +1327,7 @@ mhn_run_command (mu_message_t msg, msg_part_t part,
   return rc;
 }
 
-static RETSIGTYPE
+static void
 sigint (int sig)
 {
   /* nothing */
@@ -1333,7 +1338,7 @@ mhn_pause (void)
 {
   char c;
   int rc = 0;
-  RETSIGTYPE (*saved_sig) (int) = signal (SIGINT, sigint);
+  void (*saved_sig) (int) = signal (SIGINT, sigint);
 
   printf (_("Press <return> to show content..."));
   fflush (stdout);
@@ -2406,7 +2411,7 @@ mhn_edit (struct compose_env *env, int level)
   mu_stream_t output = NULL;
   mu_message_t msg = NULL;
   size_t line_count = 0;
-  int ascii_buf;
+  int ascii_buf = 0;
   
   while (status == 0
 	 && mu_stream_getline (env->input, &buf, &bufsize, &n) == 0 && n > 0)
