@@ -1,6 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2003, 2008-2012, 2014-2017 Free Software Foundation,
-   Inc.
+   Copyright (C) 2003-2023 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -52,12 +51,12 @@ static struct mu_option options[] = {
 int
 main (int argc, char **argv)
 {
-  mh_whatnow_env_from_environ (&wh_env);
-
+  mh_whatnow_env_from_environ_early (&wh_env);
   mh_getopt (&argc, &argv, options, 0, args_doc, prog_doc, NULL);
+  mh_whatnow_env_from_environ_late (&wh_env);
   
   if (argc)
-    wh_env.draftfile = argv[0];
+    wh_env.file = argv[0];
   else if (draftfolder)
     {
       if (mh_draft_message (draftfolder, draftmessage, &wh_env.file))
