@@ -1,5 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2009-2012, 2014-2017 Free Software Foundation, Inc.
+   Copyright (C) 2009-2023 Free Software Foundation, Inc.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -25,49 +25,18 @@ _repr (PyObject *self)
 {
   char buf[80];
   sprintf (buf, "<" PY_MODULE "." PY_CSNAME " instance at %p>", self);
-  return PyString_FromString (buf);
+  return PyUnicode_FromString (buf);
 }
 
 static PyTypeObject PySecretType = {
-  PyObject_HEAD_INIT(NULL)
-  0,                         /* ob_size */
-  PY_MODULE "." PY_CSNAME,   /* tp_name */
-  sizeof (PySecret),         /* tp_basicsize */
-  0,                         /* tp_itemsize */
-  (destructor)_py_dealloc,   /* tp_dealloc */
-  0,                         /* tp_print */
-  0,                         /* tp_getattr; __getattr__ */
-  0,                         /* tp_setattr; __setattr__ */
-  0,                         /* tp_compare; __cmp__ */
-  _repr,                     /* tp_repr; __repr__ */
-  0,                         /* tp_as_number */
-  0,                         /* tp_as_sequence */
-  0,                         /* tp_as_mapping */
-  0,                         /* tp_hash; __hash__ */
-  0,                         /* tp_call; __call__ */
-  _repr,                     /* tp_str; __str__ */
-  0,                         /* tp_getattro */
-  0,                         /* tp_setattro */
-  0,                         /* tp_as_buffer */
-  Py_TPFLAGS_DEFAULT,        /* tp_flags */
-  "",                        /* tp_doc */
-  0,                         /* tp_traverse */
-  0,                         /* tp_clear */
-  0,                         /* tp_richcompare */
-  0,                         /* tp_weaklistoffset */
-  0,                         /* tp_iter */
-  0,                         /* tp_iternext */
-  0,                         /* tp_methods */
-  0,                         /* tp_members */
-  0,                         /* tp_getset */
-  0,                         /* tp_base */
-  0,                         /* tp_dict */
-  0,                         /* tp_descr_get */
-  0,                         /* tp_descr_set */
-  0,                         /* tp_dictoffset */
-  0,                         /* tp_init */
-  0,                         /* tp_alloc */
-  0,                         /* tp_new */
+  .ob_base = { PyObject_HEAD_INIT(NULL) },
+  .tp_name = PY_MODULE "." PY_CSNAME,
+  .tp_basicsize = sizeof (PySecret),
+  .tp_dealloc = (destructor)_py_dealloc,
+  .tp_repr = _repr,
+  .tp_str = _repr,
+  .tp_flags = Py_TPFLAGS_DEFAULT,
+  .tp_doc = "",
 };
 
 PySecret *
@@ -99,7 +68,7 @@ api_secret_create (PyObject *self, PyObject *args)
       return NULL;
     }
   status = mu_secret_create (&py_secret->secret, str, len);
-  return _ro (PyInt_FromLong (status));
+  return _ro (PyLong_FromLong (status));
 }
 
 static PyObject *
@@ -124,7 +93,7 @@ api_secret_password (PyObject *self, PyObject *args)
     return NULL;
 
   pass = mu_secret_password (py_secret->secret);
-  return _ro (PyString_FromString (mu_prstr (pass)));
+  return _ro (PyUnicode_FromString (mu_prstr (pass)));
 }
 
 static PyObject *
@@ -171,6 +140,14 @@ static PyMethodDef methods[] = {
   { NULL, NULL, 0, NULL }
 };
 
+static struct PyModuleDef moduledef = {
+  PyModuleDef_HEAD_INIT,
+  PY_MODULE,
+  NULL,
+  -1,
+  methods
+};
+
 int
 mu_py_init_secret (void)
 {
@@ -182,7 +159,7 @@ void
 _mu_py_attach_secret (void)
 {
   PyObject *m;
-  if ((m = _mu_py_attach_module (PY_MODULE, methods)))
+  if ((m = _mu_py_attach_module (&moduledef)))
     {
       Py_INCREF (&PySecretType);
       PyModule_AddObject (m, PY_CSNAME, (PyObject *)&PySecretType);
