@@ -1,5 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2004-2019 Free Software Foundation, Inc.
+   Copyright (C) 2004-2023 Free Software Foundation, Inc.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -111,7 +111,7 @@ get_sql_entry (int type)
 int
 mu_sql_connection_init (mu_sql_connection_t *pconn, int interface,
 			char *server, int  port, char *login,
-			char *password, char *dbname)
+			char *password, char *dbname, char *param)
 {
   static mu_sql_dispatch_t *tab;
   mu_sql_connection_t conn;
@@ -130,6 +130,7 @@ mu_sql_connection_init (mu_sql_connection_t *pconn, int interface,
   conn->login = login;
   conn->password = password;
   conn->dbname = dbname;
+  conn->param = param;
   if (tab->init)
     {
       int rc = tab->init (conn);
